# Generated by Django 3.2.9 on 2023-12-18 12:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simo_sonos', '0002_rename_master_sonosplayer_slave_of'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sonosplaylist',
            name='item_id',
            field=models.CharField(max_length=200),
        ),
    ]
